# -*- coding: utf-8 -*-
"""

    General helpers

"""

import os, imp, sys, codecs
import traceback

from hashlib import sha512
from os.path import join as pathjoin

import jinja2
import sh

def skel_clone( outpath='my-new-project', repo='https://github.com/ionelmc/projectskel', vars=None):
    """ Clone a fucking skeleton """
    cl      = sh.git( 'clone', '--depth', '1', repo, outpath )
    gitrepo = sh.git.bake( _cwd=outpath )
    return gitrepo


def skel_addremote(gitrepo, remote_url):
    """
        Add a new remote to skeleton
    """
    return gitrepo.remote('add','origin', remote_url)


def skel_commit(gitrepo, msg):
    """ Make a  commit """
    return gitrepo.commit('-m',msg)



def skel_cleangit( path_of_gitrepo ):
    """ Deletes .git files, making it a clean repo """
    git_folder = pathjoin( path_of_gitrepo , '.git' )
    if os.path.exists( git_folder ):
        sh.rm('-rf', git_folder )


def skel_cleansurplus( project_path ):
    """
        Clean surplus files after successfull setup.
    """
    files = [ pathjoin( project_path , '__init__.py') , pathjoin( project_path, '__init__.pyc' ) ]
    return [ sh.rm('-rf', f_path) for f_path in files if os.path.exists( f_path ) ]



def render_file( tmpl, to='', skelvars=None):
    """
        Perform a jinja2 render of <file_in>, saving as <file_out>.
        If <file_in> == <file_out>, the original file will be overwrited.
    """
    file_out        = os.path.abspath( to )

    # Render the template
    content = tmpl.render( **skelvars )

    # Write to file as UTF8
    fp_out = codecs.open( file_out, 'w', 'utf-8')
    fp_out.write( content )
    fp_out.close()

    return True


def has_installed( program ):
    """
        Check if %program is installed on the machine.
    """
    try:
        sh.Command(program)
        return True
    except:
        return False



def install( program , manager='apt-get'):
    """
        Try to install program using the given manager
    """
    managers = { 'apt-get': ('apt-get','install'),
                 'brew':    ('brew','install'),
                 'python':  ('pip', 'install'),
                 'node':    ('npm', 'install')}

    m = managers[ manager ]
    sh.Command(m[0], m[1], program )


def make_password( length=16 ):
    """ Create a random password """
    return sha512( os.urandom(16) ).hexdigest()[:length]


def askYN(default=False, prompt='Continue?'):
    """
        Ask (prompt) if user may want this option. Only supports Y/N as
        answer, bool.
    """
    valid = {"yes": "yes",  "y": "yes", "ye": "yes", "t": "yes",  "true": "yes",
             "no":  "no",   "n": "no",  "f": "no", "false": "no"}

    opts = (default and '[Y/n]' or '[y/N]')
    def ask_func():
        while 1:
            sys.stdout.write('%s %s' % (prompt , opts))
            choice = raw_input().strip().lower()
            if default is not None and choice == '':
                return default
            elif choice in valid.keys():
                v = (valid[choice] == "yes" and True or False)
                return v
            else:
                sys.stdout.write("Please respond with 'yes' or 'no' "\
                                 "(or 'y' or 'n').\n")

    return ask_func



def askSTR(default='', prompt='Heheh'):
    """
        Asks for some string...
    """
    def ask_func():
        sys.stdout.write('%s' % prompt)
        return raw_input().strip().lower()

    return ask_func



#!/usr/bin/env python
import os,sys

import optparse
import blessings

parser = optparse.OptionParser(usage='%prog [options]')
term = blessings.Terminal()

from dlskel.conf    import load_config, save_config, bootstrap_dlskel
from dlskel.skel    import skeleton_clone, skeleton_runsetup


# Some globals for now
SKEL_CONFIG = '~/.dl-skel/config.json'
SKEL_PATH   = '~/.dl-skel/'

# Do some bootstrap (Loading config first )
conf = None




# Skeleton config / update
parser.add_option('', '--skeleton-config',   dest='skelconfig',       action='store', default=SKEL_CONFIG, help=term.bold('dlskel Config file. Default is %s' % term.magenta(SKEL_CONFIG)))
parser.add_option('', '--skeleton-path',     dest='skelpath',         action='store', default=SKEL_PATH,   help=term.bold('Use another Skeleton Search Path. Default is %s' % term.magenta(SKEL_PATH)))

# Options
parser.add_option('-l', '--list',     dest='list_mode',         action='store_true',    help=term.bold('List skeletons'))



# Import group
import_arg_group = optparse.OptionGroup( parser, 'Import new Template')
import_arg_group.add_option('-i', '--import',  dest='import_template',   action='store', nargs=2, metavar='<GIT_REPO> <TEMPLATE_NAME>', help=term.bold("import a new template from the given GIT repository."))

# Create new project group
create_arg_group  = optparse.OptionGroup( parser, 'Create a new project')
create_arg_group.add_option('-c', '--create',   dest='create_project',      action='store', nargs=2, metavar='<TEMPLATE_NAME> <OUTPUT>',   help=term.bold("Create a new <project> skeleton"))





def parse_args():
    parser.add_option_group(import_arg_group)
    parser.add_option_group(create_arg_group)

    (opts,args) = parser.parse_args()
    global SKEL_CONFIG, SKEL_PATH, conf

    # Overwrite config paths
    if opts.skelconfig:
        SKEL_CONFIG = opts.skelconfig
    elif opts.skelpath:
        SKEL_PATH = opts.skelpath

    # Bootstrap (if necessary)
    try:
        conf = load_config( SKEL_CONFIG )
    except:
        print 'Bootstrapping config at %s' % term.magenta( SKEL_PATH )
        bootstrap_dlskel( SKEL_PATH )
        conf = load_config( SKEL_CONFIG )

    # launch actions
    if opts.list_mode:
        list_skeletons()

    # import a new
    elif opts.import_template:

        #
        (repo_url, template_name) = opts.import_template

        # Lets see if its local or remote
        if os.path.isdir(repo_url):
            repo_url = os.path.abspath( repo_url )

        new_skel = dict(name=template_name, url = repo_url )

        all_skeletons = conf.get("SKELETONS")
        all_skeletons.append( new_skel )

        conf.upsert('SKELETONS', all_skeletons)
        save_config( SKEL_CONFIG )
        print "Imported %s sucessfully"  % (term.bold(template_name))
        #print conf.export('json', indent=4)

    elif opts.create_project is not None:

        (projname,outdir) = opts.create_project
        outdir            = os.path.abspath(outdir)

        skel = get_skel_metadata( projname )
        if not skel:
            print "Skeleton template %s does not exist..." %  term.magenta(opts.project)
            sys.exit(-1)

        print "Cloning %s(%s) into %s" % (term.magenta(skel['name']), term.underline(skel['url']), term.bold(outdir))
        repo = skeleton_clone( skel['url'] , outdir )

        # Now we do the substitutions/other steps
        skel_vars = conf.get('CONFIG', dict())
        (success, tb) = skeleton_runsetup( outdir, skel_vars=skel_vars )

        if not success:
            print '%s\n%s' % (term.red('ERROR!!'), tb )
        else:
            print '%s\n' % (term.green('SUCCESS!!'))


    else:
        print parser.print_help()



def list_skeletons():
    """ List available skeletons """
    skeleton_list = conf.get('SKELETONS')
    for skel in skeleton_list:
        print '%s\t\t%s\t%s' % ( term.magenta(skel.get('name','')) , \
                                 term.underline(skel.get('url','')) , \
                                 term.bold( skel.get('description','') ) )




def get_skel_metadata(name):
    """
        Given a name, returns the skeleton metadata.
        Returns None if skeleton not found
    """

    skel = [skel for skel in conf.get('SKELETONS') if skel['name'] == name]
    return (skel and skel[0] or None)





if __name__ == '__main__':
    parse_args()



from kaptan import Kaptan
from blessings import Terminal

import os
from os.path import join, dirname, exists, expanduser

DEFAULT_CONFIG =  {
    'SKELETONS': [
        {
         'name':        'wordpress5',
         'description': 'Wordpress 5.0 Skeleton',
         'url':         'git@github.com:doubleleft/skeleton-wordpress.git'
        },
        {
         'name':        'htmlrichapp',
         'description': 'HTML Rich App (grunt/bower/stylus)',
         'url':         'git@github.com:doubleleft/skeleton-htmlrichapp.git'
        }
    ]
}

conf = Kaptan(handler='json')
t = Terminal()

# Load config
def load_config(conf_file):
    """
        Load the conf file, returns a DICT or None
    """
    fp = expanduser( conf_file )

    conf.import_config( fp )

    return conf

def save_config( conf_file ):
    """
        Save the config file
    """
    fp = expanduser( conf_file )
    f = file( fp , 'w' )
    json = conf.export('json', indent=4,)
    f.write( json )
    f.close()


def bootstrap_dlskel( skel_path , force=False):
    """
        Create the basic structure of dlskel folder
    """
    fp = expanduser( skel_path )
    folder_exists =  exists( fp )
    if not folder_exists:
        os.makedirs( fp )

    # Create the folder to store skeletons
    skel_exists = exists( join(fp, 'skeletons'))
    if not skel_exists:
        os.makedirs( join( fp , 'skeletons' ) )

    # Create basic config
    if not exists( join(fp, 'config.json') ):
        conf.import_config(DEFAULT_CONFIG)
        save_config( join(fp, 'config.json') )



# -*- coding: utf-8 -*-
""" dlskel setup.py script """

# dl-skel
from dlskel import __version__

# system
#try:
from setuptools import setup
#except ImportError:
#    from distutils.core import setup
from os.path import join, dirname


setup(
    name='dl-skel',
    version=__version__,
    description='DoubleLeft skeleton generator',
    author='vitorfaleiros',
    author_email='barrabin.fc@gmail.com',
    packages=['dlskel'],
    scripts=['bin/dl-skel.py'],
    url='http://doubleleft.github.com/dl-skel',
    long_description=open('README.md').read(),
    install_requires=['kaptan','blessings','sh','jinja2','requests'],
    #test_suite='dlskel.test',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python',
      ],
)



# -*- coding: utf-8 -*-

import sh
import traceback

import os,sys
import imp

from .helpers import skel_clone, skel_cleansurplus, skel_cleangit

from jinja2 import FileSystemLoader, Environment

from blessings import Terminal

term = Terminal()

def skeleton_create( skeleton,
                     project,
                     skel_vars=None ):
    """
        Create a project using the given <skeleton>,
        saving it as <project>, and using
        vars as the configuration.

        Returns a tuple containing (success, description)
    """
    project_abspath = os.path.abspath( project )
    skelvars        = skel_vars or dict()

    # If local git Repository
    if os.path.isdir( skeleton ):
        skeleton = os.path.abspath( skeleton )

    # Clone
    print "Cloning %s into %s" % (term.underline(skel['url']), term.bold(project_abspath))
    skeleton_clone( skeleton , project_abspath )

    # Make the setup
    (success,tb ) = skeleton_runsetup( project_abspath , skel_vars=skel_vars )

    if not success:
        print tb
        print "%s\n%s" % (term.red('ERROR!!'), tb )
    else:
        print "%s\n" % (term.green('SUCCESS!!'))

    return (success, tb)




def skeleton_clone( skeleton, project ):
    """
        Perform a clean clone of <skeleton>, saving it as <project>
    """
    # Make a clone
    np_repository = skel_clone( outpath=project, repo=skeleton )

    # Clean git repo
    skel_cleangit( project )

    return np_repository






def skeleton_runsetup( project_abspath, skel_vars=None):
    """

        Run the setup of the skeleton. Returns a tuple
        containing (Success, Description)

    """
    skel_vars = skel_vars or dict()

    (success,tb) = (True, '')
    try:
        py_package = ( os.path.join( project_abspath, '__init__.py') )
        if not os.path.exists( py_package ):
            sh.touch( py_package )

        # Get the module
        skel_setup_file = os.path.join( project_abspath, 'dlbuild.py')
        skel_setup_mod  = imp.load_source( 'skel_setup', skel_setup_file )

        # Get the SKELETON_SETTINGS, and merge with defaults
        skel_vars_default = {'SKEL_ROOT': project_abspath,
                             'PROJECT':   os.path.split( project_abspath)[-1] }
        skel_vars         = dict()
        try:
            skel_vars_module = skel_setup_mod.SKELETON_SETTINGS

            for k,v in skel_vars_module.iteritems():
                if callable(v):
                    skel_vars_module[k] = v()

            # Merge skel_vars_default with skel_vars
            skel_vars = dict( skel_vars_default.items() + skel_vars_module.items() )

        except AtrributeError:
            skel_vars = dict( skel_vars_default.items() )

        # Create a jinja2 Loader/Context
        jinja_loader = FileSystemLoader( project_abspath )
        jinja_env    = Environment( loader=jinja_loader )

        # CD To new proj dir
        pwd = os.getcwd()
        os.chdir( project_abspath )

        # Join skel_vars
        # Run function setup with <SKEL_VARS>
        skel_setup_mod.setup( skel_vars=skel_vars , jinja_env=jinja_env )

        # Clean files
        skel_cleansurplus( project_abspath )

        # back2back
        os.chdir( pwd )

    except IOError:
        success = True
        tb  = 'Skipped skeleton setup.py...'
    except:
        success = False
        tb = traceback.format_exc()
    else:
        success = True
        tb = 'Running skeleton setup... SUCCESS'
    finally:
        return (success, tb)
